@@ -8,7 +8,7 @@
 // Software.
 
 use crate::dev_utils::{create_ids, PeerId};
-use crate::key_gen::{message::Message, Error, KeyGen};
+use crate::key_gen::{message::Message, Error, KeyGen, MessageAndTarget};
 use anyhow::{format_err, Result};
 use bincode::serialize;
 use blsttc::{PublicKeySet, SignatureShare};
@@ -18,8 +18,8 @@ use std::collections::{BTreeMap, BTreeSet};
 use xor_name::XorName;
 
 // Alter the configure of the number of nodes and the threshold.
-const NODENUM: usize = 7;
-const THRESHOLD: usize = 5;
+const NODENUM: usize = 5;
+const THRESHOLD: usize = 3;
 
 fn setup_generators<R: RngCore>(
     mut rng: &mut R,
@@ -48,7 +48,7 @@ fn create_generators<R: RngCore>(
     let mut proposals = Vec::new();
     for peer_id in peer_ids.iter() {
         let key_gen = {
-            let (key_gen, proposal) =
+            let (key_gen, messaging) =
                 match KeyGen::initialize(peer_id.name(), threshold, names.clone()) {
                     Ok(result) => result,
                     Err(err) => {
@@ -59,7 +59,7 @@ fn create_generators<R: RngCore>(
                         ))
                     }
                 };
-            proposals.push(proposal);
+            proposals.extend(messaging);
             key_gen
         };
 
@@ -74,18 +74,38 @@ fn create_generators<R: RngCore>(
 fn messaging<R: RngCore>(
     mut rng: &mut R,
     generators: &mut Vec<KeyGen>,
-    proposals: &mut Vec<Message>,
+    proposals: &mut Vec<MessageAndTarget>,
     non_responsives: BTreeSet<u64>,
 ) {
+    // Simulating the AE pattern
+    let mut cached_msg = BTreeMap::<XorName, Message>::new();
+
     // Keep broadcasting the proposals among the generators till no more.
     // The proposal from non_responsive nodes shall be ignored.
     while !proposals.is_empty() {
         let proposals_local = std::mem::take(proposals);
-        for proposal in &proposals_local {
+        for (receiver, proposal) in &proposals_local {
+            match proposal {
+                Message::Initialization { .. } | Message::Proposal { .. } => {
+                    let _ = cached_msg.insert(proposal.id(), proposal.clone());
+                }
+                _ => {}
+            }
             for (index, generator) in generators.iter_mut().enumerate() {
-                if let Ok(proposal_vec) = generator.handle_message(&mut rng, proposal.clone()) {
+                if receiver == &generator.our_id {
+                    let messaging_vec = if let Ok(messaging_vec) =
+                        generator.handle_message(&mut rng, proposal.clone())
+                    {
+                        messaging_vec
+                    } else {
+                        let mut messages: Vec<Message> = cached_msg.values().cloned().collect();
+                        messages.push(proposal.clone());
+                        let (messaging_vec, _unhandable) =
+                            generator.handle_pre_session_messages(&mut rng, messages);
+                        messaging_vec
+                    };
                     if !non_responsives.contains(&(index as u64)) {
-                        proposal_vec
+                        messaging_vec
                             .iter()
                             .for_each(|prop| proposals.push(prop.clone()));
                     }
@@ -234,7 +254,11 @@ fn having_min_unresponsive_nodes_cause_block() -> Result<()> {
         if let Err(err) = generators[index].timed_phase_transition(&mut rng) {
             assert_eq!(err, Error::TooManyNonVoters(non_responsives.clone()));
         } else {
-            return Err(format_err!("Node {:?} shall not progress anymore", peer_id));
+            return Err(format_err!(
+                "Node {:?}-{:?} shall not progress anymore",
+                index,
+                peer_id
+            ));
         }
     }
     // List already returned within the above call to `finalize_complaining_phase`. So here it
